@@ -147,7 +147,7 @@ bool Block::copy_block( Buffer & buffer, const Point & p ) throw()
   bool done = buffer.pputb( p2, tmp, tmp.bof(), tmp.eof() );
   buffer.reset_appendable();
   if( done )
-    { if( RC::options().auto_unmark ) reset(); else set_block( buffer, p, p2 ); }
+    { if( RC::editor_options().auto_unmark ) reset(); else set_block( buffer, p, p2 ); }
   return done;
   }
 
@@ -183,6 +183,6 @@ bool Block::move_block( Buffer & buffer, const Point & p ) throw()
   bool done = buffer.pputb( p2, tmp, tmp.bof(), tmp.eof() );
   buffer.reset_appendable();
   if( done )
-    { if( RC::options().auto_unmark ) reset(); else set_block( buffer, p, p2 ); }
+    { if( RC::editor_options().auto_unmark ) reset(); else set_block( buffer, p, p2 ); }
   return done;
   }
