@@ -78,7 +78,7 @@ bool Buffer::Change_history::Record::append_record( const Basic_buffer & buffer,
     { a1.end = a2.end; return true; }		// consecutive insert
   if( a1.bbp && a2.bbp )
     {
-    if( a1.begin == a1.end && a2.begin == a2.end )
+    if( a1.begin == a1.end && a2.begin == a2.end )	// two deletes
       {
       if( a1.begin == a2.begin )		// delete at same place
         {
@@ -95,7 +95,11 @@ bool Buffer::Change_history::Record::append_record( const Basic_buffer & buffer,
       else a1.begin = a1.end;
       }
     else if( a1.begin < a1.end && a2.begin < a2.end && a1.end == a2.begin )
-      { add_atom( a2 ); return true; }		// consecutive overwrite
+      {						// consecutive overwrite
+      Point p = a1.bbp->eof();
+      a1.bbp->pputb( p, *a2.bbp, a2.bbp->bof(), a2.bbp->eof() );
+      a1.end = a2.end; delete a2.bbp; a2.bbp = 0; return true;
+      }
     }
   return false;
   }
@@ -226,7 +230,7 @@ bool Buffer::save( const std::string * namep, bool set_name, bool append,
     }
   if( !namep ) return false;
 
-  if( same_name && !saved && RC::options().backup )
+  if( same_name && !saved && RC::editor_options().backup )
     {
     std::string backname = *namep; backname += '~';
     std::remove( backname.c_str() );
@@ -234,7 +238,7 @@ bool Buffer::save( const std::string * namep, bool set_name, bool append,
     }
 
   FILE *f = 0;
-  if( File::make_path( *namep ) )
+  if( Menu_file::make_path( *namep ) )
     f = std::fopen( namep->c_str(), append ? "a" : "w" );
   if( !f )
     { if( !had_name ) { _name.clear(); } throw Error( "Error opening file" ); }
@@ -353,17 +357,18 @@ bool Buffer::pputc( Point & p, const unsigned char ch ) throw()
     }
   else
     {
-    const Point p0 = p;
-    if( options.overwrite && p != eol( p ) )
+    if( options.overwrite && p < eol( p ) )
       {
-      tmp.pputc( dummy, (*this)[p] );
+      const Point p1 = Point( p.line, p.col + 1 );
+      Change_history::Record r( p, p1, _modified );
+      Change_history::Atom a( p, p1, *this );
       Basic_buffer::pchgc( p, ch );
-      Change_history::Record r( p0, p, _modified );
-      r.add_atom( Change_history::Atom( p0, p, tmp, tmp.bof(), tmp.eof() ) );
+      a.end = p1; r.add_atom( a );
       change_history.add_record( *this, r );
       }
     else
       {
+      const Point p0 = p;
       Basic_buffer::pputc( p, ch );
       Change_history::Record r( p0, p, _modified );
       r.add_atom( Change_history::Atom( p0, p ) );
@@ -449,6 +454,25 @@ bool Buffer::pdell( Point & p ) throw()
   }
 
 
+// replace block at [p1,p2) with b.[bp1,bp2) and move p2 next
+bool Buffer::replace( const Point & p1, Point & p2, const Basic_buffer & b,
+                      const Point & bp1, const Point & bp2 ) throw()
+  {
+  Change_history::Record r( p1, p1, _modified );
+  Change_history::Atom a( p1, p2, *this );
+  if( !pdelb( p1, p2, 0 ) && a.bbp ) { delete a.bbp; a.bbp = 0; }
+  p2 = p1;
+  if( pputb( p2, b, bp1, bp2, 0 ) ) a.end = p2;
+  if( a.begin < a.end || a.bbp )
+    {
+    r.add_atom( a ); change_history.add_record( *this, r );
+    _modified = true;
+    return true;
+    }
+  return false;
+  }
+
+
 bool Buffer::undo( Point & p ) throw()
   {
   if( change_history.index < 0 ) return false;
@@ -479,8 +503,7 @@ bool Buffer::undo( Point & p ) throw()
 
 bool Buffer::redo( Point & p ) throw()
   {
-  if( change_history.index + 1 >= change_history.records() )
-    return false;
+  if( change_history.index + 1 >= change_history.records() ) return false;
   ++change_history.index;
   Change_history::Record & rec = change_history.record_vector[change_history.index];
   for( unsigned int i = 0; i < rec.atom_vector.size(); ++i )
@@ -517,7 +540,7 @@ Bufhandle_vector::~Bufhandle_vector() throw()
   if( i >= handles() ) return;
 
   const std::string dead_name = "DEADMOE";
-  RC::options().backup = false;
+  RC::editor_options().backup = false;
   FILE *f = std::fopen( dead_name.c_str(), "a" );
   time_t dummy = time( 0 );
   std::fprintf( f, "\n*** Modified files in Moe when it aborted on %s", ctime( &dummy ) );
@@ -565,7 +588,7 @@ int Bufhandle_vector::find_or_add_handle( const Buffer::Options & opts,
   if( namep && namep->size() )
     for( int j = 0; j < handles(); ++j )
       if( *namep == data[j].buffer().name() )
-        return ( j + handles() - _first ) % handles();
+        return circular_index( j );
 
   Buffer *bufp = new Buffer( opts, namep );
   if( namep && namep->size() ) RC::apply_regex_options( *bufp );
@@ -573,11 +596,11 @@ int Bufhandle_vector::find_or_add_handle( const Buffer::Options & opts,
   if( i < 0 ) i = handles();
   else
     {
-    i = ( _first + i ) % handles();
+    i = absolute_index( i );
     if( i == 0 ) i = handles(); else if( i <= _first ) ++_first;
     }
   data.insert( data.begin() + i, Buffer_handle( *bufp, line ) );
-  return ( i + handles() - _first ) % handles();
+  return circular_index( i );
   }
 
 
@@ -591,7 +614,7 @@ void Bufhandle_vector::add_handle_if_pending_input_or_empty( const int line ) th
 
 void Bufhandle_vector::duplicate_handle( int i ) throw()
   {
-  i = ( _first + i ) % handles();
+  i = absolute_index( i );
   if( i < _first ) ++_first;
   data.insert( data.begin() + i + 1, data[i] );
   }
@@ -600,7 +623,7 @@ void Bufhandle_vector::duplicate_handle( int i ) throw()
 bool Bufhandle_vector::delete_handle( int i ) throw()
   {
   if( i < 0 || i >= handles() ) return false;
-  else i = ( _first + i ) % handles();
+  else i = absolute_index( i );
   Buffer & buffer = data[i].buffer();
   int j;
   for( j = 0; j < handles(); ++j )
