@@ -135,20 +135,16 @@ class Buffer : public Basic_buffer
       Point end;		// Buffer position of end of insert
       Basic_buffer *bbp;	// Basic_buffer containing deleted data
 
-      // Creates an insert Atom
+      // Creates an insert Atom ([p1,p2) -> inserted_text)
       Atom( const Point & p1, const Point & p2 ) throw()
         : begin( p1 ), end( p2 ), bbp( 0 ) {}
 
-      // Creates a delete Atom
+      // Creates a delete Atom (bbp -> deleted_text)
       Atom( const Point & p1, const Point & p2, const Basic_buffer & b ) throw()
-        : begin( p1 ), end( p1 )
-        { bbp = new Basic_buffer( b, p1, p2 ); }
+        : begin( p1 ), end( p1 ), bbp( 0 )
+        { if( p1 < p2 ) bbp = new Basic_buffer( b, p1, p2 ); }
 
-      // Creates a change (overwrite) Atom
-      Atom( const Point & p1, const Point & p2, const Basic_buffer & b,
-            const Point & bp1, const Point & bp2 ) throw()
-        : begin( p1 ), end( p2 )
-        { bbp = new Basic_buffer( b, bp1, bp2 ); }
+      // To create a replace Atom make a delete Atom, then move end.
       };
 
     struct Record
@@ -227,6 +223,10 @@ public:
   bool pdelc( Point & p, const bool back = false ) throw();
   bool pdell( Point & p ) throw();	// delete line and set 'p' to bol()
 
+       // replace block at [p1,p2) with b.[bp1,bp2) and move p2 next
+  bool replace( const Point & p1, Point & p2, const Basic_buffer & b,
+                const Point & bp1, const Point & bp2 ) throw();
+
   bool undo( Point & p ) throw();
   bool redo( Point & p ) throw();
   void force_append() const throw() { change_history.force_append = true; }
@@ -265,8 +265,12 @@ public:
   ~Bufhandle_vector() throw();
   bool save_all_named() const throw();
 
-  Buffer_handle & operator[]( int i ) throw()
-    { return data[( _first + i ) % handles()]; }
+  int absolute_index( const int i ) const throw()
+    { return ( i + _first ) % handles(); }
+  int circular_index( const int i ) const throw()
+    { return ( i + handles() - _first ) % handles(); }
+  Buffer_handle & operator[]( const int i ) throw()
+    { return data[absolute_index(i)]; }
   int handles() const throw() { return data.size(); }
   int handles( const Buffer & buffer ) const throw();
   void next() throw() { _first = ++_first % handles(); }
