@@ -39,25 +39,16 @@
 
 namespace {
 
-// Date of this version: 2006-01-28
+// Date of this version: 2006-04-03
 
 const char * invocation_name = 0;
 const char * const Program_name     = "GNU Moe";
 const char * const program_name     = "moe";
 const char * const config_file_name = "moerc";
-const char * const program_version  = "0.6";
+const char * const program_version  = "0.7";
 const char * const program_year     = "2006";
 
 
-void show_version() throw()
-  {
-  std::printf( "%s version %s\n", Program_name, program_version );
-  std::printf( "Copyright (C) %s Antonio Diaz Diaz.\n", program_year );
-  std::printf( "This program is free software; you may redistribute it under the terms of\n" );
-  std::printf( "the GNU General Public License.  This program has absolutely no warranty.\n" );
-  }
-
-
 void show_help() throw()
   {
   std::printf( "%s - A powerful and user-friendly text editor.\n", Program_name );
@@ -67,27 +58,36 @@ void show_help() throw()
   std::printf( "Editor options:\n" );
   std::printf( "  -h, --help               display this help and exit\n" );
   std::printf( "  -V, --version            output version information and exit\n" );
-  std::printf( "  -u, --auto-unmark        Turn off highlighting after block copy or move\n" );
   std::printf( "  -b, --backup             Create backup files (default)\n" );
   std::printf( "  -e, --exit-ask           Save-and-close always asks for confirmation\n" );
+  std::printf( "  -H, --smart-home         Go home, then go to first non-blank character\n" );
   std::printf( "  -i, --ignore-case        Make search case insensitive by default\n" );
   std::printf( "  -k, --keep-lines=<n>     Number of lines to keep for PgUp/PgDn\n" );
   std::printf( "      --orphan             Put extra files in orphaned buffers\n" );
 //  std::printf( "  -p, --beep               Beep on errors and when cursor goes past extremes\n" );
-//  std::printf( "  -x, --rectangle          Rectangular block mode\n" );
   std::printf( "  -s, --search-wrap        Search wraps at end of file\n" );
-  std::printf( "  -H, --smart-home         Go home, then go to first non-blank character\n" );
+  std::printf( "  -u, --auto-unmark        Turn off highlighting after block copy or move\n" );
+//  std::printf( "  -x, --rectangle          Rectangular block mode\n" );
   std::printf( "File options:\n" );
   std::printf( "  -a, --auto-indent        Auto indent\n" );
   std::printf( "  -l, --lmargin=<col>      Left margin\n" );
   std::printf( "  -r, --rmargin=<col>      Right margin\n" );
-  std::printf( "  -O, --overwrite          Insert mode off\n" );
   std::printf( "  -o, --read-only          File is read-only\n" );
+  std::printf( "  -O, --overwrite          Insert mode off\n" );
   std::printf( "  -w, --word-wrap          Word wrap\n" );
   std::printf( "\nReport bugs to bug-moe@gnu.org\n" );
   }
 
 
+void show_version() throw()
+  {
+  std::printf( "%s version %s\n", Program_name, program_version );
+  std::printf( "Copyright (C) %s Antonio Diaz Diaz.\n", program_year );
+  std::printf( "This program is free software; you may redistribute it under the terms of\n" );
+  std::printf( "the GNU General Public License.  This program has absolutely no warranty.\n" );
+  }
+
+
 int set_signal( int signum, void (*handler)(int) ) throw()
   {
   struct sigaction new_action;
@@ -176,15 +176,15 @@ void RC::show_error( const char * msg, const int errcode, const bool help ) thro
   }
 
 
-int main( int argc, char *argv[] ) throw()
+int main( int argc, char * argv[] ) throw()
   {
   invocation_name = argv[0];
 
   for( int i = 1; i < argc; ++i )
     {
-    if( !strcmp( argv[i], "-h" ) || !strcmp( argv[i], "--help" ) )
+    if( !std::strcmp( argv[i], "-h" ) || !std::strcmp( argv[i], "--help" ) )
       { show_help(); return 0; }
-    if( !strcmp( argv[i], "-V" ) || !strcmp( argv[i], "--version" ) )
+    if( !std::strcmp( argv[i], "-V" ) || !std::strcmp( argv[i], "--version" ) )
       { show_version(); return 0; }
     }
 
@@ -200,7 +200,6 @@ int main( int argc, char *argv[] ) throw()
       std::fprintf( stderr, "Press RETURN to continue, Ctrl-C to abort. " );
       std::fgetc( stdin ); std::fprintf( stderr, "\n" ); retval = 0;
       }
-    else if( retval ) RC::show_error( 0, 0, true );
     }
   if( retval ) return retval;
 
