@@ -16,21 +16,24 @@
     Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
 */
 
-namespace File {
+namespace Menu_file {
+
+// Returns handle index, or -1 if aborted.
+//
+int bufhandle_menu( int current, const int abort_key ) throw();
 
 // 'name' contains the filename to be modified (may be empty).
 // Returns false if aborted.
 //
 bool file_menu( const std::string & prompt, std::string & name ) throw();
 
-
 // 'name' contains the default result (may be empty), and
 // stores the result (if any).
 // Returns the size of the result string, 0 if none, -1 if aborted.
 // 'name' is cleared if aborted.
 //
-int get_filename( const std::string & prompt, std::string & name ) throw();
-
+int get_filename( const std::string & prompt, std::string & name,
+                  const int abort_key = 3 ) throw();
 
 bool is_regular( const std::string & name, const bool quiet = false ) throw();
 
@@ -42,18 +45,18 @@ bool test_slash_filename( std::string & name ) throw();
 
 bool tilde_expansion( std::string & name ) throw();
 
-} // end namespace File
+} // end namespace Menu_file
 
 
-namespace Help {
+namespace Menu_help {
 
-void help_menu( int help_key ) throw();
+void help_menu( const int abort_key ) throw();
 
-} // end namespace Help
+} // end namespace Menu_help
 
 
-namespace Menu {
+namespace Menu_opts {
 
-void options_menu( Buffer & buffer, const int menu_key ) throw();
+std::string options_menu( Buffer & buffer, const int abort_key ) throw();
 
-} // end namespace Menu
+} // end namespace Menu_opts
