@@ -31,7 +31,7 @@
 #include "screen.h"
 
 
-namespace File {
+namespace Menu_file {
 
 void show( const std::vector< std::string > & name_vector, const int lines,
            const int columns, const int current, Point & cursor ) throw()
@@ -114,13 +114,89 @@ bool valid_filename( const std::string & name ) throw()
   return ( basename.size() && basename != "." && basename != ".." );
   }
 
-} // end namespace File
+} // end namespace Menu_file
+
+
+// Returns handle index, or -1 if aborted.
+//
+int Menu_file::bufhandle_menu( int current, const int abort_key ) throw()
+  {
+  std::vector< std::string > name_vector;
+
+  const int entries = bufhandle_vector.handles();
+  for( int i = 0; i < entries; ++i )
+    {
+    const int c = bufhandle_vector.circular_index( i );
+    name_vector.push_back( bufhandle_vector[c].buffer().uname() );
+    }
+
+  if( entries < 2 || entries != (int)name_vector.size() ) return -1;
+
+  int max_width = 0;
+  for( int i = 0; i < entries; ++i )
+    if( (int)name_vector[i].size() > max_width )
+      max_width = name_vector[i].size();
+  if( max_width == 0 ) return -1;
+
+  const int columns = std::max( 1, Screen::width() / ( max_width + 2 ) );
+  const int _height = ( entries + columns - 1 ) / columns;
+
+  const int lines = std::min( 10, _height ), line = Screen::height() - lines - 1;
+  Point cursor( 0, Screen::width() - 1 );
+
+  Screen::save_lines_and_cursor( line, lines + 1 );
+
+  Screen::out_line( "Select buffer handle (^C to abort)", Screen::height() - 1 );
+  current = bufhandle_vector.absolute_index( current );
+  int key = 0;
+  while( key >= 0 )
+    {
+    show( name_vector, lines, columns, current, cursor );
+    Screen::move_to( Point( line, 0 ) + cursor );
+    bool valid_key = false;
+    while( !valid_key )
+      {
+      valid_key = true;
+      do key = Screen::wait_kbhit( line ); while( key < 0 );
+      if( key == abort_key ) key = 3;
+      switch( key )
+        {
+        case 3   : key = -1; break;		// ^C
+        case ' ' :
+        case '\r': key = -2; break;
+        default: switch( Screen::convert_key( key ) )
+                   {
+                   case Screen::key_up   :
+                     if( current >= columns ) current -= columns; break;
+                   case Screen::key_down :
+                     if( current + columns < entries ) current += columns; break;
+                   case Screen::key_left : if( current > 0 ) --current; break;
+                   case Screen::key_right: if( current < entries - 1 ) ++current; break;
+                   case Screen::key_ppage:
+                     for( int i = 0; i < lines && current >= columns; ++i )
+                       current -= columns;
+                     break;
+                   case Screen::key_npage:
+                     for( int i = 0; i < lines && current + columns < entries; ++i )
+                       current += columns;
+                     break;
+                   case Screen::key_home : current = 0; break;
+                   case Screen::key_end  : current = entries - 1; break;
+                   default: valid_key = false;
+                   }
+        }
+      }
+    }
+  Screen::restore_lines_and_cursor();
+  if( key == -1 ) return -1;
+  else return bufhandle_vector.circular_index( current );
+  }
 
 
 // 'name' contains the filename to be modified (may be empty).
 // Returns false if aborted.
 //
-bool File::file_menu( const std::string & prompt, std::string & name ) throw()
+bool Menu_file::file_menu( const std::string & prompt, std::string & name ) throw()
   {
   std::string dirname, basename;
   std::vector< std::string > name_vector;
@@ -176,6 +252,7 @@ bool File::file_menu( const std::string & prompt, std::string & name ) throw()
         {
         case 3   : key = -1; break;		// ^C
         case '\t': key = -2; break;
+        case ' ' :
         case '\r': key = -3; name = dirname;
                    append_part_or_go_up( name, name_vector[current] );
                    break;
@@ -214,19 +291,20 @@ bool File::file_menu( const std::string & prompt, std::string & name ) throw()
 // Returns the size of the result string, 0 if none, -1 if aborted.
 // 'name' is cleared if aborted.
 //
-int File::get_filename( const std::string & prompt, std::string & name ) throw()
+int Menu_file::get_filename( const std::string & prompt, std::string & name,
+                             const int abort_key ) throw()
   {
   static std::vector< std::string > file_history;
 
   file_history.push_back( name );
-  int len = Screen::get_string( prompt, file_history, false, true );
+  int len = Screen::get_string( prompt, file_history, abort_key, false, true );
   if( len > 0 ) name = file_history.back();
   else name.clear();
   return len;
   }
 
 
-bool File::is_regular( const std::string & name, const bool quiet ) throw()
+bool Menu_file::is_regular( const std::string & name, const bool quiet ) throw()
   {
   if( name.size() && name[name.size()-1] != '/' )
     {
@@ -240,7 +318,7 @@ bool File::is_regular( const std::string & name, const bool quiet ) throw()
   }
 
 
-bool File::make_path( const std::string & name ) throw()
+bool Menu_file::make_path( const std::string & name ) throw()
   {
   std::string dirname, basename;
   split_filename( name, dirname, basename );
@@ -266,7 +344,7 @@ bool File::make_path( const std::string & name ) throw()
   }
 
 
-const std::string & File::my_basename( const std::string & name ) throw()
+const std::string & Menu_file::my_basename( const std::string & name ) throw()
   {
   static std::string basename;
   std::string dirname;
@@ -275,7 +353,7 @@ const std::string & File::my_basename( const std::string & name ) throw()
   }
 
 
-bool File::test_slash_filename( std::string & name ) throw()
+bool Menu_file::test_slash_filename( std::string & name ) throw()
   {
   tilde_expansion( name );
   return ( !name.size() ||
@@ -283,7 +361,7 @@ bool File::test_slash_filename( std::string & name ) throw()
   }
 
 
-bool File::tilde_expansion( std::string & name ) throw()
+bool Menu_file::tilde_expansion( std::string & name ) throw()
   {
   if( name.size() >= 2 && !name.compare( 0, 2, "~/" ) )
     { name.replace( 0, 1, RC::home_directory() ); return true; }
