@@ -24,23 +24,23 @@
 #include "screen.h"
 
 
-namespace Help {
+namespace Menu_help {
 
 const char *help_line[] = {
   "@i Help Screen: Keyboard Commands               PAGE_DN next         F1 exit help",
   " @bF1@b Help             @bF5@b Prev window     @bF9@b Copy block   ^C  Close window",
   " @bF2@b Save file        @bF6@b Next window    @bF10@b Options      ^X  Save and close",
-  " @bF3@b Load file        @bF7@b Undo           @bF11@b              ^QX Save all and exit",
+  " @bF3@b Load file        @bF7@b Undo           @bF11@b Buffers      ^QX Save all and exit",
   " @bF4@b Zoom window      @bF8@b Redo           @bF12@b              ^QC Exit without saving",
   " ",
   " @bGO TO@b               @bBLOCK@b             @bSEARCH@b           @bMISC@b",
   " ^[U Beg of file     ^SPACE Mark       ^F Find          Ins Insert/Overwrite",
-  " ^[V End of file     ^KB Begin         ^L Find next     Tab Filename completion",
+  " ^[V End of file     ^KB Begin         ^G Find next     Tab Filename completion",
   " ^[L Line            ^KK End                            ^B  Reformat paragraph",
   " ^[C Column          ^KC Copy          @bDELETE@b           ^P  Control char",
   " ^[B Beg of block    ^KM Move          ^Y Line          ^S  Show status",
   " ^[K End of block    ^KY Delete                         ^OC Center line",
-  " ^[G Match pair ->   ^KR Read                           ^OR Refresh",
+  " ^[G Match pair ->   ^KR Read                           ^OR Repaint screen",
   " ^[H Match pair <-   ^KW Write                          ^OS Split window",
   " ^[D Center cursor   ^KI Indent                         ^OV Show version",
   "                     ^KU Unindent                       ^Q@bF2@b Save named buffers",
@@ -112,23 +112,23 @@ void show_help( int delta ) throw()
   while( line < Screen::height() ) { Screen::out_line( "", line ); ++line; }
   }
 
-} // end namespace Help
+} // end namespace Menu_help
 
 
-void Help::help_menu( int help_key ) throw()
+void Menu_help::help_menu( const int abort_key ) throw()
   {
   Screen::save_lines_and_cursor( 0, Screen::height() );
 
-  int key = help_key + 1, delta = 0;
-  while( key != help_key )
+  int key = abort_key + 1, delta = 0;
+  while( key != abort_key )
     {
     show_help( delta );
     Screen::move_to( Point( 0, Screen::width() - 1 ) );
     while( true )
       {
       key = Screen::wait_kbhit( 1 );		// show clock on line 0 only
-      if( key == help_key ) break;
-      if( key == 3 ) { key = help_key; break; }		// ^C
+      if( key == 3 ) key = abort_key;		// ^C
+      if( key == abort_key ) break;
       if( Screen::convert_key( key ) == Screen::key_npage ) { delta = +1; break; }
       if( Screen::convert_key( key ) == Screen::key_ppage ) { delta = -1; break; }
       }
