@@ -22,6 +22,7 @@
 #include <string>
 #include <vector>
 
+#include "arg_parser.h"
 #include "buffer.h"
 #include "menu.h"
 #include "rc.h"
@@ -65,146 +66,84 @@ std::vector< Regex_options > regex_options_vector;
 
 enum Optcode
   {
-  auto_unmark, no_auto_unmark, backup, no_backup, /*beep, no_beep,*/
-  exit_ask, no_exit_ask, ignore_case, no_ignore_case, keep_lines,
-  orphan_extra, /*rectangle_mode, no_rectangle_mode,*/ search_wrap,
-  no_search_wrap, smart_home, no_smart_home, auto_indent,
-  no_auto_indent, lmargin, rmargin, overwrite, no_overwrite, read_only,
-  no_read_only, word_wrap, no_word_wrap,
-  END_OPTS
+  auto_unmark = 'u', backup = 'b', /*beep = 'p',*/ exit_ask = 'e',
+  ignore_case = 'i', keep_lines = 'k', /*rectangle_mode = 'x',*/
+  search_wrap = 's', smart_home = 'H', auto_indent = 'a', lmargin = 'l',
+  rmargin = 'r', overwrite = 'O', read_only = 'o', word_wrap = 'w',
+  no_auto_unmark = 256, no_backup, /*no_beep,*/ no_exit_ask, no_ignore_case,
+  orphan_extra, /*no_rectangle_mode,*/ no_search_wrap, no_smart_home,
+  no_auto_indent, no_overwrite, no_read_only, no_word_wrap,
   };
 
-
-struct Entry
+const Arg_parser::Option options[] =
   {
-  Optcode optcode;		// Option code
-  const char * name;		// Option name
-  };
-
-Entry table[] = {
-    { auto_unmark,       "-u" },
-    { auto_unmark,       "--auto-unmark" },
-    { no_auto_unmark,    "--no-auto-unmark" },
-    { backup,            "-b" },
-    { backup,            "--backup" },
-    { no_backup,         "--no-backup" },
-//    { beep,              "-p" },
-//    { beep,              "--beep" },
-//    { no_beep,           "--no-beep" },
-    { exit_ask,          "-e" },
-    { exit_ask,          "--exit-ask" },
-    { no_exit_ask,       "--no-exit-ask" },
-    { ignore_case,       "-i" },
-    { ignore_case,       "--ignore-case" },
-    { no_ignore_case,    "--no-ignore-case" },
-    { keep_lines,        "-k" },
-    { keep_lines,        "--keep-lines" },
-    { orphan_extra,      "--orphan" },
-//    { rectangle_mode,    "-x" },
-//    { rectangle_mode,    "--rectangle" },
-//    { no_rectangle_mode, "--no-rectangle" },
-    { search_wrap,       "-s" },
-    { search_wrap,       "--search-wrap" },
-    { no_search_wrap,    "--no-search-wrap" },
-    { smart_home,        "-H" },
-    { smart_home,        "--smart-home" },
-    { no_smart_home,     "--no-smart-home" },
-    { auto_indent,       "-a" },
-    { auto_indent,       "--auto-indent" },
-    { no_auto_indent,    "--no-auto-indent" },
-    { lmargin,           "-l" },
-    { lmargin,           "--lmargin" },
-    { rmargin,           "-r" },
-    { rmargin,           "--rmargin" },
-    { overwrite,         "-O" },
-    { overwrite,         "--overwrite" },
-    { no_overwrite,      "--no-overwrite" },
-    { read_only,         "-o" },
-    { read_only,         "--read-only" },
-    { no_read_only,      "--no-read-only" },
-    { word_wrap,         "-w" },
-    { word_wrap,         "--word-wrap" },
-    { no_word_wrap,      "--no-word-wrap" },
-    { END_OPTS, 0 } };
+  { auto_unmark,       "auto-unmark",    Arg_parser::no  },
+  { no_auto_unmark,    "no-auto-unmark", Arg_parser::no  },
+  { backup,            "backup",         Arg_parser::no  },
+  { no_backup,         "no-backup",      Arg_parser::no  },
+//  { beep,              "beep",           Arg_parser::no  },
+//  { no_beep,           "no-beep",        Arg_parser::no  },
+  { exit_ask,          "exit-ask",       Arg_parser::no  },
+  { no_exit_ask,       "no-exit-ask",    Arg_parser::no  },
+  { ignore_case,       "ignore-case",    Arg_parser::no  },
+  { no_ignore_case,    "no-ignore-case", Arg_parser::no  },
+  { keep_lines,        "keep-lines",     Arg_parser::yes },
+  { orphan_extra,      "orphan",         Arg_parser::no  },
+//  { rectangle_mode,    "rectangle",      Arg_parser::no  },
+//  { no_rectangle_mode, "no-rectangle",   Arg_parser::no  },
+  { search_wrap,       "search-wrap",    Arg_parser::no  },
+  { no_search_wrap,    "no-search-wrap", Arg_parser::no  },
+  { smart_home,        "smart-home",     Arg_parser::no  },
+  { no_smart_home,     "no-smart-home",  Arg_parser::no  },
+  { auto_indent,       "auto-indent",    Arg_parser::no  },
+  { no_auto_indent,    "no-auto-indent", Arg_parser::no  },
+  { lmargin,           "lmargin",        Arg_parser::yes },
+  { rmargin,           "rmargin",        Arg_parser::yes },
+  { overwrite,         "overwrite",      Arg_parser::no  },
+  { no_overwrite,      "no-overwrite",   Arg_parser::no  },
+  { read_only,         "read-only",      Arg_parser::no  },
+  { no_read_only,      "no-read-only",   Arg_parser::no  },
+  { word_wrap,         "word-wrap",      Arg_parser::no  },
+  { no_word_wrap,      "no-word-wrap",   Arg_parser::no  },
+  { 0,                 0,                Arg_parser::no  } };
 
 
-Optcode to_optcode( const char *optname ) throw()
+const char * optname( const int code ) throw()
   {
-  for( int i = 0; table[i].name; ++i )
-    if( strcmp( optname, table[i].name ) == 0 )
-      return table[i].optcode;
-  return END_OPTS;
+  static char buf[2] = "?";
+
+  if( code != 0 )
+    for( int i = 0; options[i].code; ++i )
+      if( code == options[i].code )
+        { if( options[i].name ) return options[i].name; else break; }
+  if( code > 0 && code < 256 ) buf[0] = code; else buf[0] = '?';
+  return buf;
   }
 
 
-bool is_editor_option( Optcode optcode ) throw()
+bool is_buffer_option( const int code ) throw()
   {
-  switch( optcode )
+  switch( code )
     {
-    case auto_unmark:
-    case no_auto_unmark:
-    case backup:
-    case no_backup:
-//    case beep:
-//    case no_beep:
-    case exit_ask:
-    case no_exit_ask:
-    case ignore_case:
-    case no_ignore_case:
-    case keep_lines:
-    case orphan_extra:
-//    case rectangle_mode:
-//    case no_rectangle_mode:
-    case smart_home:
-    case no_smart_home:
-    case search_wrap:
-    case no_search_wrap:
-         return true;
-    case auto_indent:
+    case auto_indent   :
     case no_auto_indent:
-    case lmargin:
-    case rmargin:
-    case overwrite:
-    case no_overwrite:
-    case read_only:
-    case no_read_only:
-    case word_wrap:
-    case no_word_wrap:
-    case END_OPTS:
-         return false;
-    }
-  return false;		// keep compiler happy
-  }
-
-
-bool is_buffer_option( Optcode optcode ) throw()
-  {
-  return ( !is_editor_option( optcode ) && optcode != END_OPTS );
-  }
-
-
-bool optarg_included( char **optname, char **optarg ) throw()
-  {
-  static char buf[3];
-  char *name = *optname;
-  if( name[0] == '-' )
-    {
-    if( name[1] == '-' )
-      {
-      for( int i = 2; name[i]; ++i )
-        if( name[i] == '=' )
-          { name[i] = 0; *optarg = name + i + 1; return true; }
-      }
-    else if( name[1] != 0 && name[2] != 0 )
-      {
-      buf[0] = '-'; buf[1] = name[1]; buf[2] = 0;
-      *optname = buf; *optarg = name + 2; return true;
-      }
+    case lmargin       :
+    case rmargin       :
+    case overwrite     :
+    case no_overwrite  :
+    case read_only     :
+    case no_read_only  :
+    case word_wrap     :
+    case no_word_wrap  : return true;
     }
   return false;
   }
 
 
+bool is_editor_option( const int code ) throw()
+  { return ( !is_buffer_option( code ) ); }
+
+
 void set_bool_option( bool & option, bool value,
                       int & option_priority, int priority = 0 ) throw()
   {
@@ -213,31 +152,28 @@ void set_bool_option( bool & option, bool value,
   }
 
 
-bool set_int_option( int & option, const char *optarg,
-                      int & option_priority, int priority = 0 ) throw()
+bool set_int_option( int & option, const std::string & arg,
+                     int & option_priority, int priority = 0 ) throw()
   {
   bool good = true;
-  if( ( !priority || priority >= option_priority ) && optarg )
-    good = parse_int( optarg, option );
+  if( ( !priority || priority >= option_priority ) && arg.size() )
+    good = parse_int( arg, option );
   if( priority > option_priority ) option_priority = priority;
   return good;
   }
 
 
-// return value: no. of fields taken (1 or 2), or 0 if option not found.
-//
-int set_option_from_name( char *optname, char *optarg,
-                          Options *edop, Buffer::Options *bufop,
-                          int priority = 0, char *msg = "" )
+bool set_option_from_code( const int code, const std::string & arg,
+                           Options *edop, Buffer::Options *bufop,
+                           int priority = 0, char *msg = "" )
   {
-  bool arg_included = optarg_included( &optname, &optarg );
-  bool arg_needed = false;
-  Optcode optcode = to_optcode( optname );
+  if( ( is_editor_option( code ) && !edop ) ||
+      ( is_buffer_option( code ) && !bufop ) )
+    { std::fprintf( stderr, "%soption `%s' not allowed here\n", msg, optname( code ) );
+      return false; }
 
   int errcode = 0;
-  if( ( is_editor_option( optcode ) && !edop ) ||
-      ( is_buffer_option( optcode ) && !bufop ) ) errcode = 2;
-  else switch( optcode )
+  switch( code )
     {
     case auto_unmark:       edop->auto_unmark = true; break;
     case no_auto_unmark:    edop->auto_unmark = false; break;
@@ -249,8 +185,8 @@ int set_option_from_name( char *optname, char *optarg,
     case no_exit_ask:       edop->exit_ask = false; break;
     case ignore_case:       edop->ignore_case = true; break;
     case no_ignore_case:    edop->ignore_case = false; break;
-    case keep_lines:        if( !parse_int( optarg, edop->keep_lines ) ) errcode = 5;
-                            arg_needed = true; break;
+    case keep_lines:        if( !parse_int( arg, edop->keep_lines ) ) errcode = 1;
+                            break;
     case orphan_extra:      edop->orphan_extra = true; break;
 //    case rectangle_mode:    edop->rectangle_mode = true; break;
 //    case no_rectangle_mode: edop->rectangle_mode = false; break;
@@ -265,16 +201,14 @@ int set_option_from_name( char *optname, char *optarg,
     case lmargin:
       {
       int i;
-      if( !set_int_option( i, optarg, priorities.lmargin, priority ) ) errcode = 5;
-      else if( !bufop->set_lmargin( i - 1 ) ) errcode = 6;
-      arg_needed = true;
+      if( !set_int_option( i, arg, priorities.lmargin, priority ) ) errcode = 1;
+      else if( !bufop->set_lmargin( i - 1 ) ) errcode = 2;
       } break;
     case rmargin:
       {
       int i;
-      if( !set_int_option( i, optarg, priorities.rmargin, priority ) ) errcode = 5;
-      else if( !bufop->set_rmargin( i - 1 ) ) errcode = 6;
-      arg_needed = true;
+      if( !set_int_option( i, arg, priorities.rmargin, priority ) ) errcode = 1;
+      else if( !bufop->set_rmargin( i - 1 ) ) errcode = 2;
       } break;
     case overwrite:
       set_bool_option( bufop->overwrite, true, priorities.overwrite, priority ); break;
@@ -288,36 +222,22 @@ int set_option_from_name( char *optname, char *optarg,
       set_bool_option( bufop->word_wrap, true, priorities.word_wrap, priority ); break;
     case no_word_wrap:
       set_bool_option( bufop->word_wrap, false, priorities.word_wrap, priority ); break;
-    case END_OPTS: errcode = 1; break;
+    default: std::fprintf( stderr, "%sinternal error: uncaught option\n", msg );
+      return false;
     }
 
-  if( !errcode )
-    {
-    if( arg_needed ) { if( !optarg || !*optarg ) errcode = 3; }
-    else if( arg_included ) errcode = 4;
-    }
-  switch( errcode )
-    {
-    case 0: if( !arg_needed || arg_included ) return 1; else return 2;
-    case 1: std::fprintf( stderr, "%sUnknown option %s\n", msg, optname );
-            break;
-    case 2: std::fprintf( stderr, "%sOption %s not allowed here\n", msg, optname );
-            break;
-    case 3: std::fprintf( stderr, "%sOption %s needs an argument\n", msg, optname );
-            break;
-    case 4: std::fprintf( stderr, "%sOption %s doesn't admit an argument\n", msg, optname );
-            break;
-    case 5: std::fprintf( stderr, "%sBad argument for option %s\n", msg, optname );
-            break;
-    case 6: std::fprintf( stderr, "%sValue out of range for option %s\n", msg, optname );
-    }
-  return 0;
+  if( !errcode ) return true;
+  if( errcode == 1 )
+    std::fprintf( stderr, "%sbad argument for option `%s'\n", msg, optname( code ) );
+  else if( errcode == 2 )
+    std::fprintf( stderr, "%svalue out of range for option `%s'\n", msg, optname( code ) );
+  return false;
   }
 
 } // end namespace RC
 
 
-RC::Options & RC::options() throw() { return _editor_options; }
+RC::Options & RC::editor_options() throw() { return _editor_options; }
 
 
 const Buffer::Options & RC::default_buffer_options() throw()
@@ -331,7 +251,7 @@ void RC::apply_regex_options( Buffer & buffer ) throw()
     const Regex_options & ro = regex_options_vector[i];
     unsigned int j = 0;
     for( ; j < ro.regex_vector.size(); ++j )
-      if( Regex::match_filename( ro.regex_vector[j], File::my_basename( buffer.name() ) ) )
+      if( Regex::match_filename( ro.regex_vector[j], Menu_file::my_basename( buffer.name() ) ) )
         break;
     if( j >= ro.regex_vector.size() ) continue;
     if( priorities.lmargin <= 2 )
@@ -361,14 +281,13 @@ bool RC::parse_int( const std::string & s, int & result ) throw()
   }
 
 
-// Returns 0 for success, 1 for file not found, 2 for syntax error.
-//
+    // Returns 0 for success, 1 for file not found, 2 for syntax error.
 int RC::process_rcfile( const std::string & name ) throw()
   {
   FILE *f = std::fopen( name.c_str(), "r" );
   if( !f ) return 1;
 
-  std::fprintf( stderr, "Processing '%s'... ", name.c_str() );
+  std::fprintf( stderr, "Processing `%s'... ", name.c_str() );
   std::fflush( stderr );
 
   const char delimiters[] = " \t\r\n";
@@ -392,16 +311,20 @@ int RC::process_rcfile( const std::string & name ) throw()
       }
     else			// Set an option
       {
-      char *optname = strtok( buf, delimiters );
-      char *optarg = strtok( 0, delimiters );
+      char *opt = strtok( buf, delimiters );
+      char *arg = strtok( 0, delimiters );
       char msg[80];
       std::snprintf( msg, sizeof(msg), "\n%s %d: ", name.c_str(), line );
-      if( regex_status == 0 )
-        { if( !set_option_from_name( optname, optarg, &_editor_options,
-                                     &_default_buffer_options, 1, msg ) ) retval = 2; }
+      Arg_parser parser( opt, arg, options );
+      if( parser.error().size() || !parser.arguments() || !parser.code( 0 ) )
+        { std::fprintf( stderr, "%s%s\n", msg, parser.error().c_str() ); retval = 2; }
+      else if( regex_status == 0 )
+        { if( !set_option_from_code( parser.code( 0 ), parser.argument( 0 ),
+                                     &_editor_options, &_default_buffer_options,
+                                     1, msg ) ) retval = 2; }
       else
         {
-        if( !set_option_from_name( optname, optarg, 0,
+        if( !set_option_from_code( parser.code( 0 ), parser.argument( 0 ), 0,
                                    &regex_options_vector.back().buffer_options,
                                    0, msg ) ) retval = 2;
         regex_status = 2;
@@ -415,65 +338,67 @@ int RC::process_rcfile( const std::string & name ) throw()
   }
 
 
-// Returns 0 for success, 1 for invalid option, 2 if non-regular files given.
-//
-int RC::process_options( int argc, char *argv[] ) throw()
+    // Returns 0 for success, 1 for invalid option, 2 if non-regular files given.
+int RC::process_options( const int argc, const char * const argv[] ) throw()
   {
-  int line = 0, optind;
-  bool non_regular_found = false;
-
   std::fprintf( stderr, "Processing options... " );
   std::fflush( stderr );
 
-  for( optind = 1; optind < argc; ++optind )	// Process "global" options
-    {
-    if( argv[optind][0] == '+' )
+  Arg_parser parser( argc, argv, options, true );
+  if( parser.error().size() )				// bad option
+    { show_error( parser.error().c_str(), 0, true ); return 1; }
+
+  int argind, line = 0;
+  for( argind = 0; argind < parser.arguments(); ++argind )
+    {						// Process "global" options
+    const int code = parser.code( argind );
+    const std::string & arg = parser.argument( argind );
+    if( !code )
       {
-      if( parse_int( argv[optind] + 1, line ) && --line >= 0 ) continue;
-      else { std::fprintf( stderr, "Bad line number\n" ); return 1; }
+      if( arg[0] != '+' ) break;			// filename
+      if( parse_int( arg, line ) && --line >= 0 ) continue;
+      show_error( "bad line number" ); return 1;
       }
-    if( argv[optind][0] != '-' || !argv[optind][1] ) break;
-    const int n = set_option_from_name( argv[optind], argv[optind+1],
-                                        &_editor_options,
-                                        &_default_buffer_options, 3 );
-    if( n == 0 ) return 1; else if( n == 2 ) ++optind;
+    if( !set_option_from_code( code, arg, &_editor_options,
+                               &_default_buffer_options, 3 ) ) return 1;
     }
 
-  try
-    {
-    while( optind < argc )	// Process input files and file options
+  bool non_regular_found = false;
+  try {					// Process input files and file options
+    while( argind < parser.arguments() )
       {
-      std::string name = argv[optind++];
-      File::tilde_expansion( name );
-      const bool regular = File::is_regular( name, true );
+      std::string name = parser.argument( argind++ );
+      Menu_file::tilde_expansion( name );
+      const bool regular = Menu_file::is_regular( name, true );
       if( !regular )
         {
-        std::fprintf( stderr, "Warning: `%s' is not a regular file\n", name.c_str() );
+        std::fprintf( stderr, "warning: `%s' is not a regular file\n", name.c_str() );
         name.clear(); non_regular_found = true;
         }
       const int i = bufhandle_vector.find_or_add_handle( _default_buffer_options, &name, -1, line );
       line = 0;
       Buffer & buffer = bufhandle_vector[i].buffer();
-      for( ; optind < argc; ++optind )		// Process file options
+      for( ; argind < parser.arguments(); ++argind )	// Process file options
         {
-        if( argv[optind][0] == '+' )
+        const int code = parser.code( argind );
+        const std::string & arg = parser.argument( argind );
+        if( !code )
           {
-          if( parse_int( argv[optind] + 1, line ) && --line >= 0 ) continue;
-          else { std::fprintf( stderr, "Bad line number\n" ); return 1; }
+          if( arg[0] != '+' ) break;			// filename
+          if( parse_int( arg, line ) && --line >= 0 ) continue;
+          show_error( "bad line number" ); return 1;
           }
-        if( argv[optind][0] != '-' || !argv[optind][1] ) break;
-        const int n = set_option_from_name( argv[optind], argv[optind+1],
-                                            0, &buffer.options );
-        if( n == 0 ) return 1; else if( n == 2 ) ++optind;
+        if( !set_option_from_code( code, parser.argument( argind ),
+                                   0, &buffer.options ) ) return 1;
         }
       if( !regular ) bufhandle_vector.delete_handle( i );
       }
     }
-  catch( Buffer::Error e ) { std::fprintf( stderr, "%s\n", e.s ); return 1; }
+  catch( Buffer::Error e ) { show_error( e.s ); return 1; }
 
   // Add empty buffer if no files given or stdin is not a tty
   try { bufhandle_vector.add_handle_if_pending_input_or_empty( line ); }
-  catch( Buffer::Error e ) { std::fprintf( stderr, "%s\n", e.s ); return 1; }
+  catch( Buffer::Error e ) { show_error( e.s ); return 1; }
 
   std::fprintf( stderr, "done\n" );
   if( non_regular_found ) return 2; else return 0;
