@@ -73,17 +73,32 @@ void rep_addch( int n, const chtype ch )
   { for( ; n > 0; --n ) waddch( stdscr, ch ); }
 
 
+void out_raw_string( const int line, const int col, const std::string & s,
+                     const int len, int pos = 0, const chtype atr = A_NORMAL ) throw()
+  {
+  wmove( stdscr, line, col );
+  int x = col, limit = std::min( x + len, _width );
+  for( ; x < limit && pos < (int)s.size(); ++pos, ++x )
+    {
+    unsigned char ch = s[pos];
+    waddch( stdscr, atr | ch );
+    }
+  if( x < limit ) rep_addch( limit - x, atr | ' ' );
+  }
+
+
 // '@b' sets-resets bold mode
 // '@i' sets-resets inverse video mode
 // '@u' sets-resets underline mode
 // '@@' shows a '@'
 //
-void out_string( const std::string & s, int len, int pos = 0 ) throw()
+void out_string( const int line, const int col, const std::string & s,
+                 const int len, int pos = 0 ) throw()
   {
-  if( len > _width ) len = _width;
+  wmove( stdscr, line, col );
+  int x = col, limit = std::min( x + len, _width );
   chtype atr = A_NORMAL;
-  int x;
-  for( x = 0; x < len && pos < (int)s.size(); ++pos )
+  for( ; x < limit && pos < (int)s.size(); ++pos )
     {
     unsigned char ch = s[pos];
     if( ch == '@' && pos + 1 < (int)s.size() )
@@ -96,7 +111,7 @@ void out_string( const std::string & s, int len, int pos = 0 ) throw()
         }
     if( ch ) { waddch( stdscr, atr | ch ); ++x; }
     }
-  if( x < len ) rep_addch( len - x, atr | ' ' );
+  if( x < limit ) rep_addch( limit - x, atr | ' ' );
   }
 
 
@@ -172,7 +187,8 @@ void control_o_loop( int key, const std::string & version_msg ) throw()
   switch( key )
     {
     case 'C': window_vector.center_line(); break;
-    case 'R': window_vector.repaint(); break;
+    case 'R': werase( stdscr ); wrefresh( stdscr );
+              window_vector.repaint(); break;
     case 'S': window_vector.split_window(); break;
     case 'V': show_message( version_msg ); break;
     }
@@ -201,7 +217,7 @@ void user_control_char() throw()
   int key;
   while( true )
     { key = show_message( prompt, true, true );
-      if( key == KEY_F(1) ) Help::help_menu( key ); else break; }
+      if( key == KEY_F(1) ) Menu_help::help_menu( key ); else break; }
   if( key < 0 || key >= 256 || key == 3 ) return;
   if( key != '\\' && !std::isdigit( key ) )
     { key = ISO_8859::controlize( key ); if( key >= 32 ) key = -1; }
@@ -219,6 +235,36 @@ void user_control_char() throw()
   else window_vector.add_char( key, true );
   }
 
+
+// `read_key' implements a buffer to speed up pastes, because `wgetch'
+// always calls `wrefresh'.
+//
+int read_key( const bool wait, const bool get_key ) throw()
+  {
+  static std::vector< int > buf;
+  static unsigned int index = 0;	// index of key to be returned
+
+  if( index >= buf.size() )		// buf is empty
+    {
+    buf.clear(); index = 0;
+    if( wait )
+      {
+      const int key = wgetch( stdscr );
+      if( key == ERR ) return -1; else buf.push_back( key );
+      }
+    nodelay( stdscr, true );
+    while( true )
+      {
+      const int key = wgetch( stdscr );
+      if( key == ERR ) break; else buf.push_back( key );
+      }
+    nodelay( stdscr, false );
+    }
+  if( index < buf.size() )
+    { const int key = buf[index]; if( get_key ) ++index; return key; }
+  return -1;
+  }
+
 } // end namespace Screen
 
 
@@ -236,61 +282,57 @@ bool Screen::init() throw()
 //void Screen::beep() throw() { ::beep(); wrefresh( stdscr ); }
 
 
+void Screen::clock_handler( const int sig ) throw()	// updates the clock
+  {
+  static std::string s( " 0:00" );
+
+  time_t dummy = time( 0 ); struct tm *t = localtime( &dummy );
+  if( t )
+    {
+    int h = t->tm_hour % 12; if( h == 0 ) h = 12;
+    s[0] = ( h >= 10 ) ? '0' + ( h / 10 ) : ' '; s[1] = '0' + ( h % 10 );
+    s[2] = ( sig == 0 || s[2] != ':') ? ':' : ' ';
+    s[3] = '0' + ( t->tm_min / 10 ); s[4] = '0' + ( t->tm_min % 10 );
+    Point old_cursor; save_cursor( old_cursor );
+    if( clock_limit == 1 )
+      out_raw_string( 0, 60, s, 5, 0, A_REVERSE );
+    else for( int i = 0; i < window_vector.windows(); ++i )
+      {
+      Point cursor = window_vector[i].clock_position();
+      if( cursor.line >= clock_limit ) break;
+      out_raw_string( cursor.line, cursor.col, s, 5, 0, A_REVERSE );
+      }
+    wmove( stdscr, old_cursor.line, old_cursor.col );
+    if( sig ) { wrefresh( stdscr ); alarm( 1 ); }
+    }
+  }
+
+
+int Screen::kbhit() throw() { return read_key( false, false ); }
+
+
 // Wait for a keystroke.
 // The clock only blinks when waiting here for a keystroke.
 // The screen is also only refreshed here (and in clock_handler).
 // This function is the real "main loop" of the program.
 //
-int Screen::wait_kbhit( const int new_clock_limit, const bool get_key ) throw()
+int Screen::wait_kbhit( const int new_clock_limit, const bool get_key,
+                        const bool force_refresh ) throw()
   {
-  if( new_clock_limit < 0 )
-    move_to( window_vector.curwin().absolute_cursor() );
-
-  clock_limit = ( new_clock_limit >= 0 ) ? new_clock_limit : _height;
-  if( clock_limit > 0 ) { clock_handler( 0 ); alarm( 1 ); }
-  else wrefresh( stdscr );
-
-  int key = wgetch( stdscr );
-  alarm( 0 );
-  if( key == ERR ) return -1;
-  if( !get_key ) ungetch( key );
-  return key;
-  }
-
-
-int Screen::kbhit() throw()
-  { 
-  nodelay( stdscr, true );
-  int key = wgetch( stdscr );
-  nodelay( stdscr, false );
-  if( key == ERR ) return 0;
-  else { ungetch( key ); return key; }
-  }
-
-
-void Screen::clock_handler( const int sig ) throw()	// updates the clock
-  {
-  static char c = ':';
-
-  if( sig ) { alarm( 1 ); c = (c != ':')? ':' : ' '; } else c = ':';
-  time_t dummy = time(0); struct tm *t = localtime( &dummy );
-  if( t )
+  if( force_refresh || kbhit() < 0 )
     {
-    int h = t->tm_hour%12; if( h == 0 ) h = 12;
-    char buf[16];
-    std::snprintf( buf, sizeof( buf ), "@i%2d%c%02d", h, c, t->tm_min );
-    std::string s( buf );
-    Point old_cursor; save_cursor( old_cursor );
-    if( clock_limit == 1 ) { wmove( stdscr, 0, 60 ); out_string( s, 5 ); }
-    else for( int i = 0; i < window_vector.windows(); ++i )
-      {
-      Point cursor = window_vector[i].clock_position();
-      if( cursor.line >= clock_limit ) break;
-      move_to( cursor ); out_string( s, 5 );
-      }
-    move_to( old_cursor );
-    if( sig ) wrefresh( stdscr );
+    if( new_clock_limit < 0 )
+      move_to( window_vector.curwin().absolute_cursor() );
+
+    clock_limit = ( new_clock_limit >= 0 ) ? new_clock_limit : _height;
+    if( clock_limit > 0 ) { clock_handler( 0 ); alarm( 1 ); }
+    wrefresh( stdscr );
     }
+
+  int key = read_key( true, get_key );
+  alarm( 0 );
+  if( key < 0 ) key = -1;
+  return key;
   }
 
 
@@ -300,6 +342,7 @@ void Screen::clock_handler( const int sig ) throw()	// updates the clock
 //
 int Screen::get_string( const std::string & prompt,
                         std::vector< std::string > & history,
+                        const int abort_key,
                         const bool allow_copy, const bool usetab ) throw()
   {
   const int line = _height - 1, lines = 1;
@@ -315,27 +358,27 @@ int Screen::get_string( const std::string & prompt,
   while( key >= 0 )
     {
     int pos = scol; pos -= ( _width - 1 ); if( pos < 0 ) pos = 0;
-    wmove( stdscr, line, 0 );
-    out_string( s, _width, pos );
+    out_raw_string( line, 0, s, _width, pos );
     wmove( stdscr, line, scol - pos );
     do key = wait_kbhit( line ); while( key < 0 );
+    if( key == abort_key ) key = 3;
     switch( key )
       {
       case   3 : key = -1; break;		// ^C
       case '\t': if( usetab )
                    {
                    std::string name( s, prompt.size(), s.size() );
-                   if( !File::file_menu( prompt, name ) ) { key = -1; break; }
+                   if( !Menu_file::file_menu( prompt, name ) ) { key = -1; break; }
                    s.erase( prompt.size(), s.size() ); s += name;
                    scol = s.size(); modified = true;
                    }
                  break;
       case '\r': { std::string name( s, prompt.size(), s.size() );
                    if( !usetab || !name.size() ||
-                       ( File::test_slash_filename( name ) && File::is_regular( name ) ) )
+                       ( Menu_file::test_slash_filename( name ) && Menu_file::is_regular( name ) ) )
                      { key = -2; history.back() = name; } }
                  break;
-      case KEY_F(1) : Help::help_menu( key ); break;
+      case KEY_F(1) : Menu_help::help_menu( key ); break;
       case KEY_F(9) : if( allow_copy && Block::valid() &&
                           Block::begin().line == Block::end().line )
                         {
@@ -425,7 +468,7 @@ void Screen::restore_lines_and_cursor() throw()
   for( int y = lc.line, i = 0; y < lc.line + lc.lines; ++y )
     for( int x = 0; x < _width; ++x )
       { wmove( stdscr, y, x ); waddch( stdscr, lc.buf[i++] ); }
-  move_to( lc.cursor );
+  wmove( stdscr, lc.cursor.line, lc.cursor.col );
   lines_and_cursor_vector.pop_back();
   }
 
@@ -458,20 +501,24 @@ void Screen::out_buffer_line( const Buffer & buffer, Point c, const int line ) t
   }
 
 
-bool Screen::out_line( const std::string & s, const int line, const bool shift ) throw()
+bool Screen::out_line( const std::string & s, const int line, const bool shift,
+                       const char mode ) throw()
   {
   if( line < 0 || line >= _height ) return false;
   int pos = s.size(); pos -= ( _width - 1 );
   if( !shift || pos < 0 ) pos = 0;
-  wmove( stdscr, line, 0 );
-  out_string( s, _width, pos );
+  if( !mode ) out_string( line, 0, s, _width, pos );
+  else
+    {
+    chtype atr = A_NORMAL;
+    if( mode == 'b' ) atr = A_BOLD;
+    else if( mode == 'i' ) atr = A_REVERSE;
+    else if( mode == 'u' ) atr = A_UNDERLINE;
+    out_raw_string( line, 0, s, _width, pos, atr );
+    }
   return true;
   }
 
-/*
-void Screen::repaint( const Buffer * bufp ) throw()
-  { window_vector.repaint( bufp ); }
-*/
 
 int Screen::show_message( const std::string & s, const bool get_key,
                           const bool take_cursor ) throw()
@@ -480,7 +527,7 @@ int Screen::show_message( const std::string & s, const bool get_key,
 
   save_lines_and_cursor( line, 1 );
 
-  wmove( stdscr, line, 0 ); out_string( s, _width );
+  out_string( line, 0, s, _width );
   wmove( stdscr, line, std::min( (int)s.size(), _width - 1 ) );
   int key;
   do key = wait_kbhit( take_cursor ? line : -1, get_key ); while( key < 0 );
@@ -490,10 +537,6 @@ int Screen::show_message( const std::string & s, const bool get_key,
   }
 
 
-void Screen::show_status_lines() throw()
-  { window_vector.show_status_lines(); }
-
-
 Screen::Keys Screen::convert_key( const int key ) throw()
   {
   switch( key )
@@ -515,10 +558,12 @@ int Screen::user_loop( const std::string & version_msg ) throw()
   {
   show_message( version_msg );
   int retval = -1;
+  bool force_refresh = false;
   while( retval < 0 )
     {
     int key;
-    do key = wait_kbhit(); while( key < 0 );
+    do key = wait_kbhit( -1, true, force_refresh ); while( key < 0 );
+    force_refresh = false;
     Window & w = window_vector.curwin();
     switch( key )
       {
@@ -526,6 +571,7 @@ int Screen::user_loop( const std::string & version_msg ) throw()
       case  2: window_vector.reformat(); break;			// ^B
       case  3: if( !window_vector.close( true ) ) retval = 0; break;	// ^C
       case  6: window_vector.search(); break;			// ^F
+      case  7: window_vector.search( true ); break;		// ^G
       case 11: control_k_loop( key ); break;			// ^K
       case 12: window_vector.search( true ); break;		// ^L
       case 15: control_o_loop( key, version_msg ); break;	// ^O
@@ -535,16 +581,17 @@ int Screen::user_loop( const std::string & version_msg ) throw()
       case 24: if( !window_vector.close( false ) ) retval = 0; break;	// ^X
       case 25: window_vector.delete_line(); break;		// ^Y
       case 27: alt_key_loop( key ); break;			// Alt or Esc
-      case KEY_F(1) : Help::help_menu( key ); break;
-      case KEY_F(2) : window_vector.save_file(); break;
-      case KEY_F(3) : window_vector.load_file(); break;
+      case KEY_F(1) : Menu_help::help_menu( key ); break;
+      case KEY_F(2) : window_vector.save_file( key ); break;
+      case KEY_F(3) : window_vector.load_file( key ); break;
       case KEY_F(4) : window_vector.load( -1 ); break;
       case KEY_F(5) : window_vector.prev(); break;
       case KEY_F(6) : window_vector.next(); break;
       case KEY_F(7) : window_vector.undo(); break;
       case KEY_F(8) : window_vector.redo(); break;
       case KEY_F(9) : window_vector.copy_block(); break;
-      case KEY_F(10): Menu::options_menu( w.buffer(), key ); break;
+      case KEY_F(10): window_vector.options_menu( w.buffer(), key ); break;
+      case KEY_F(11): window_vector.bufhandle_menu( key ); break;
       case KEY_BACKSPACE: window_vector.delete_char( true ); break;
       case KEY_DC   : window_vector.delete_char(); break;
       case KEY_IC   : w.buffer().options.overwrite = !w.buffer().options.overwrite;
@@ -557,7 +604,9 @@ int Screen::user_loop( const std::string & version_msg ) throw()
       case KEY_DOWN : w.move_vertical( +1 ); break;
       case KEY_LEFT : w.goto_pprev(); break;
       case KEY_RIGHT: w.goto_pnext(); break;
-      default       : if( key < 256 ) { window_vector.add_char( key ); break; }
+      default       : if( key < 256 )
+                        { if( key == '\r' || key == '\n' ) force_refresh = true;
+                          window_vector.add_char( key ); }
       }
     if( Block::follow_marking( w.buffer(), w.buffer_handle().pointer ) )
       window_vector.repaint( &w.buffer() );
