@@ -81,7 +81,7 @@ void Window::goto_home() throw()
     p = buffer.bol( p );
   else
     {
-    if( !RC::options().smart_home || p == buffer.bot( p ) ) return;
+    if( !RC::editor_options().smart_home || p == buffer.bot( p ) ) return;
     p = buffer.bot( p );
     }
   update_points( p );
@@ -212,7 +212,7 @@ void Window::goto_pprev() throw()
 
 void Window::move_page( const bool down ) throw()
   {
-  const int keep_lines = RC::options().keep_lines;
+  const int keep_lines = RC::editor_options().keep_lines;
   int lines = ( _height - 1 ) / 2;
 
   if( keep_lines >= 0 && keep_lines <= lines )
@@ -254,7 +254,7 @@ void Window::scroll_horizontal( const int cols ) throw()
 
 void Window::scroll_page( const bool down ) throw()
   {
-  const int keep_lines = RC::options().keep_lines;
+  const int keep_lines = RC::editor_options().keep_lines;
   int lines = ( _height - 1 ) / 2;
 
   if( keep_lines >= 0 && keep_lines < lines )
@@ -315,18 +315,18 @@ void Window::show_character_info() const throw()
 void Window::show_status_line( const char * prefix ) const throw()
   {
   if( _top_line < 0 ) return;
-  char buf[Screen::width()+3];
-  int len = std::snprintf( buf, sizeof( buf ), "@i%-3s %c%c%c%c%c%c %s %s",
+  char buf[Screen::width()+1];
+  int len = std::snprintf( buf, sizeof( buf ), "%-3s %c%c%c%c%c%c %s %s",
                            prefix,
                            bhp->buffer().options.overwrite ? 'O' : 'I',
                            bhp->buffer().options.word_wrap ? 'W' : ' ',
                            bhp->buffer().options.auto_indent ? 'A' : ' ',
                            bhp->buffer().options.read_only ? 'R' : ' ',
-                           /*RC::options().rectangle_mode ? 'X' :*/ ' ',
+                           /*RC::editor_options().rectangle_mode ? 'X' :*/ ' ',
                            bhp->buffer().modified() ? '*' : ' ',
                            bhp->buffer().uname().c_str(),
                            bhp->buffer().modified() ? "(Modified)" : "" );
-  int offset = Screen::width() - 40 + 2;	// +2 for the '@i'
+  int offset = Screen::width() - 40;
   const Point & pc = bhp->pcursor;
   for( int i = pc.line + 1; i > 9999; i /= 10 ) --offset;
   for( int i = pc.col + 1; i > 999; i /= 10 ) --offset;
@@ -334,7 +334,7 @@ void Window::show_status_line( const char * prefix ) const throw()
   std::snprintf( buf + offset, sizeof( buf ) - offset,
                  "  Line %-4d Col %-3d 00:00  F1 for help",
                  pc.line + 1, pc.col + 1 );
-  Screen::out_line( buf, _top_line );
+  Screen::out_line( buf, _top_line, false, 'i' );
   }
 
 
