@@ -75,9 +75,10 @@ public:
   int init() throw();
   int load( int new_size = 0 ) throw();
 
-  void load_file() throw();
+  void bufhandle_menu( const int abort_key ) throw();
+  void load_file( const int abort_key ) throw();
   void split_window() throw();
-  void save_file() const throw();
+  void save_file( const int abort_key ) const throw();
   void save_all_named() const throw();
   int close( bool abort, const bool quiet = false ) throw();
   int close_and_exit( const bool abort ) throw();
@@ -104,6 +105,7 @@ public:
   void center_line() throw();
   void reformat() throw();
 
+  void options_menu( Buffer & buffer, const int abort_key ) const throw();
   void toggle_marking() const throw();
   void repaint( const Buffer * bufp = 0 ) const throw();
   void show_status_lines( const Buffer * bufp = 0, const char * prefix = "" ) const throw();
