@@ -69,7 +69,7 @@ void update_points( std::vector< Window > & data, const Buffer * bufp ) throw()
 int Window_vector::init() throw()
   {
   _curwin = 0;
-  return load( RC::options().orphan_extra ? 1 : bufhandle_vector.handles() );
+  return load( RC::editor_options().orphan_extra ? 1 : bufhandle_vector.handles() );
   }
 
 
@@ -110,10 +110,17 @@ int Window_vector::load( int new_size ) throw()
   }
 
 
-void Window_vector::load_file() throw()
+void Window_vector::bufhandle_menu( const int abort_key ) throw()
+  {
+  const int handle = Menu_file::bufhandle_menu( _curwin, abort_key );
+  if( handle >= 0 ) { _curwin = handle; load(); }
+  }
+
+
+void Window_vector::load_file( const int abort_key ) throw()
   {
   std::string name;
-  if( File::get_filename( "Name of file to edit (^C to abort): ", name ) < 0 )
+  if( Menu_file::get_filename( "Name of file to edit (^C to abort): ", name, abort_key ) < 0 )
     return;
   const int old_handles = bufhandle_vector.handles();
   int handle;
@@ -134,12 +141,12 @@ void Window_vector::split_window() throw()
   }
 
 
-void Window_vector::save_file() const throw()
+void Window_vector::save_file( const int abort_key ) const throw()
   {
   const Buffer & buffer = data[_curwin].buffer();
 
   std::string name = buffer.name();
-  if( File::get_filename( "Name of file to save (^C to abort): ", name ) <= 0 )
+  if( Menu_file::get_filename( "Name of file to save (^C to abort): ", name, abort_key ) <= 0 )
     return;
   bool done;
   try { done = buffer.save( &name ); }
@@ -171,7 +178,7 @@ int Window_vector::close( bool abort, const bool quiet ) throw()
   Buffer_handle & bh = w.buffer_handle();
   const bool last_handle = bufhandle_vector.handles( bh.buffer() ) <= 1;
   const bool modified = bh.buffer().modified();
-  const bool exit_ask = RC::options().exit_ask && !quiet;
+  const bool exit_ask = RC::editor_options().exit_ask && !quiet;
 
   if( !abort || !quiet )
     {
@@ -185,7 +192,7 @@ int Window_vector::close( bool abort, const bool quiet ) throw()
       {
       std::string name = bh.buffer().name();
       if( exit_ask || !name.size() )
-        while( !File::get_filename( "Name of file to save (^C to abort): ", name ) );
+        while( !Menu_file::get_filename( "Name of file to save (^C to abort): ", name ) );
       if( !name.size() ) return -1;
       try { if( !bh.buffer().save( &name ) ) return -1; }
       catch( Buffer::Error e )
@@ -326,7 +333,7 @@ void Window_vector::read_block() throw()
   if( read_only( w.buffer() ) ) return;
 
   std::string name;
-  if( File::get_filename( "Name of file to insert (^C to abort): ", name ) <= 0 )
+  if( Menu_file::get_filename( "Name of file to insert (^C to abort): ", name ) <= 0 )
     return;
   try {
     Buffer tmp( RC::default_buffer_options(), &name );
@@ -343,7 +350,7 @@ void Window_vector::write_block() const throw()
   {
   if( no_block() ) return;
   std::string name;
-  if( File::get_filename( "Name of file to write (^C to abort): ", name ) <= 0 )
+  if( Menu_file::get_filename( "Name of file to write (^C to abort): ", name ) <= 0 )
     return;
   try { Block::buffer().save( &name, false, false,
                               Block::begin(), Block::end() ); }
@@ -505,6 +512,8 @@ void Window_vector::reformat() throw()
       if( !std::isspace( ch ) )
         { tmp.pputc( dummy, ch ); dummy = tmp.eof(); space = false; }
       else if( !space ) { tmp.pputc( dummy, ' ' ); space = true; }
+      if( Screen::kbhit() >= 0 && Screen::wait_kbhit() == 3 )
+        { Screen::show_message( "Aborted" ); return; }
       }
     if( space ) tmp.pdelc( dummy, true );
     if( dummy != tmp.bol( dummy ) ) tmp.pputc( dummy, '\n' );
@@ -527,6 +536,14 @@ void Window_vector::reformat() throw()
   }
 
 
+void Window_vector::options_menu( Buffer & buffer, const int abort_key ) const throw()
+  {
+  const std::string msg = Menu_opts::options_menu( buffer, abort_key );
+  if( msg.size() )
+    { show_status_lines( &buffer ); Screen::show_message( msg ); }
+  }
+
+
 void Window_vector::toggle_marking() const throw()
   {
   const Window & w = data[_curwin];
@@ -564,18 +581,18 @@ void Window_vector::search( const bool again ) throw()
   if( !again || search_history.size() == 0 )
     {
     search_history.push_back( std::string() );
-    if( Screen::get_string( "Find (^C to abort): ", search_history, true ) <= 0 )
+    if( Screen::get_string( "Find (^C to abort): ", search_history, 3, true ) <= 0 )
       return;
     search_string = search_history.back(); ISO_8859::deescapize( search_string );
     std::string prompt;
-    if( RC::options().ignore_case )
+    if( RC::editor_options().ignore_case )
       prompt = "(N)o_ignore (R)eplace (B)ackwards (G)lobal Bloc(K) (^C to abort): ";
     else prompt = "(I)gnore (R)eplace (B)ackwards (G)lobal Bloc(K) (^C to abort): ";
     options_history.push_back( std::string() );
     const int result = Screen::get_string( prompt, options_history );
     if( result < 0 ) return;
     backward = false; block = false; global = false; replace = false;
-    icase = RC::options().ignore_case;
+    icase = RC::editor_options().ignore_case;
     if( result > 0 )
       {
       const std::string & options = options_history.back();
@@ -594,7 +611,7 @@ void Window_vector::search( const bool again ) throw()
       {
       replace_string.clear();
       replace_history.push_back( std::string() );
-      const int len = Screen::get_string( "Replace with (^C to abort): ", replace_history, true );
+      const int len = Screen::get_string( "Replace with (^C to abort): ", replace_history, 3, true );
       if( len < 0 ) { replace = false; return; }
       if( len > 0 )
         { replace_string = replace_history.back();
@@ -606,7 +623,7 @@ void Window_vector::search( const bool again ) throw()
   if( block )
     {
     if( no_block() || no_block_in_this_file( curwin().buffer() ) ) return;
-//    if( RC::options().rectangle_mode )
+//    if( RC::editor_options().rectangle_mode )
 //      { Screen::show_message( "Can't search block in rectangle mode" ); return; }
     if( !again || !Block::in_block_or_end( curwin().buffer(), p ) )
       { if( backward ) p = Block::end(); else p = Block::begin(); }
@@ -637,7 +654,7 @@ void Window_vector::search( const bool again ) throw()
         while( true )
           {
           int key = 0;
-          if( ask || Screen::kbhit() )
+          if( ask || Screen::kbhit() >= 0 )
             {
             key = Screen::show_message( "Replace (Y)es (N)o (R)est (^C to abort): ", true );
             if( key == 3 ) break;
@@ -648,7 +665,9 @@ void Window_vector::search( const bool again ) throw()
           if( !ask || key == 'y' || key == 'Y' )
             {
             Block::restore_block_position();
+            if( ask ) w.buffer().reset_appendable();
             const bool b = Regex::replace( w.buffer(), p1, p2, replace_string, regex_pieces );
+            if( ask ) w.buffer().reset_appendable();
             Block::save_block_position();
             if( b )
               { if( empty_replace && ( p1 < p2 ) ) empty_replace = false;
@@ -671,7 +690,7 @@ void Window_vector::search( const bool again ) throw()
       if( backward ) { prev(); p = curwin().buffer().eof(); }
       else { next(); p = curwin().buffer().bof(); }
       }
-    if( ( ( global && i == 0 ) || RC::options().search_wrap ) && !wrapped )
+    if( ( ( global && i == 0 ) || RC::editor_options().search_wrap ) && !wrapped )
       {
       if( backward ) p = curwin().buffer().eof();
       else p = w.buffer().bof();
